package eu.mhsl.craftattack.spawn.appliances.doubeDoor;

import eu.mhsl.craftattack.spawn.appliance.Appliance;
import eu.mhsl.craftattack.spawn.appliances.settings.Settings;
import org.bukkit.block.Block;
import org.bukkit.block.BlockFace;
import org.bukkit.block.data.BlockData;
import org.bukkit.block.data.type.Door;
import org.bukkit.event.Listener;
import org.jetbrains.annotations.NotNull;

import java.util.List;

public class DoubleDoor extends Appliance {
    @Override
    public void onEnable() {
        Settings.instance().declareSetting(DoubleDoorSetting.class);
    }

    public void openNextDoor(Block block) {
        BlockData clickedData = block.getBlockData();
        if (!(clickedData instanceof Door clickedDoor)) return;

        BlockFace neighborFace = getNeighborFace(clickedDoor.getFacing(), clickedDoor.getHinge());
        Block neighbourBlock = block.getRelative(neighborFace);
        BlockData neighbourData = neighbourBlock.getBlockData();

        if(!(neighbourData instanceof Door neighbourDoor)) return;
        if(!(neighbourDoor.getFacing() == clickedDoor.getFacing())) return;
        if(neighbourDoor.getHinge() == clickedDoor.getHinge()) return;

        neighbourDoor.setOpen(!clickedDoor.isOpen());
        neighbourBlock.setBlockData(neighbourDoor);
    }

    private @NotNull BlockFace getNeighborFace(BlockFace face, Door.Hinge hinge) {
        return switch(face) {
            case EAST -> (hinge == Door.Hinge.RIGHT) ? BlockFace.NORTH : BlockFace.SOUTH;
            case WEST -> (hinge == Door.Hinge.RIGHT) ? BlockFace.SOUTH : BlockFace.NORTH;
            case SOUTH -> (hinge == Door.Hinge.RIGHT) ? BlockFace.EAST : BlockFace.WEST;
            case NORTH -> (hinge == Door.Hinge.RIGHT) ? BlockFace.WEST : BlockFace.EAST;
            default -> throw new IllegalStateException(String.format("BlockFace '%s' of clicked door is not valid!", face));
        };
    }

    @Override
    protected @NotNull List<Listener> listeners() {
        return List.of(new OnDoorInteractListener());
    }
}
