package eu.mhsl.craftattack.spawn.appliances.optionLinks;

import eu.mhsl.craftattack.spawn.Main;
import eu.mhsl.craftattack.spawn.appliance.Appliance;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.Bukkit;
import org.bukkit.ServerLinks;
import org.bukkit.entity.Player;
import org.bukkit.event.Listener;
import org.jetbrains.annotations.NotNull;

import java.net.URI;
import java.util.List;
import java.util.function.Function;
import java.util.logging.Level;

@SuppressWarnings("UnstableApiUsage")
public class OptionLinks extends Appliance {
    record ComponentSupplier() {}
    record UriSupplier(Player player) {}
    record UriConsumer(String uri) {}
    record SuppliedLink(Function<ComponentSupplier, Component> component, Function<UriSupplier, UriConsumer> uri) {}

    List<SuppliedLink> links = List.of(
        new SuppliedLink(
            componentSupplier -> Component.text("CraftAttack Homepage", NamedTextColor.GOLD),
            uriSupplier -> new UriConsumer("https://mhsl.eu/craftattack")
        ),
        new SuppliedLink(
            componentSupplier -> Component.text("Regeln", NamedTextColor.GOLD),
            uriSupplier -> new UriConsumer("https://mhsl.eu/craftattack/rules")
        )
    );

    @Override
    public void onEnable() {
        Bukkit.getServer().getServerLinks().getLinks()
            .forEach(serverLink -> Bukkit.getServer().getServerLinks().removeLink(serverLink));
    }

    public void setServerLinks(Player player) {
        ServerLinks playerLinks = Bukkit.getServerLinks().copy();
         Bukkit.getScheduler().runTaskAsynchronously(Main.instance(), () -> {
             this.links.forEach(suppliedLink -> {
                 Component component = suppliedLink.component.apply(new ComponentSupplier());
                 String uri = suppliedLink.uri.apply(new UriSupplier(player)).uri;

                 try {
                     playerLinks.addLink(component, URI.create(uri));
                 } catch(IllegalArgumentException e) {
                     Main.logger().log(Level.INFO, String.format("Failed to create OptionLink '%s' for player '%s'", uri, player.getName()), e);
                 }
             });
             player.sendLinks(playerLinks);
         });
    }

    @Override
    protected @NotNull List<Listener> listeners() {
        return List.of(new UpdateLinksListener());
    }
}
