package eu.mhsl.craftattack.spawn.appliances.packSelect;

import eu.mhsl.craftattack.spawn.Main;
import eu.mhsl.craftattack.spawn.appliance.Appliance;
import eu.mhsl.craftattack.spawn.appliance.ApplianceCommand;
import eu.mhsl.craftattack.spawn.appliances.packSelect.listeners.ClosePackInventoryListener;
import eu.mhsl.craftattack.spawn.appliances.packSelect.listeners.ClickPackInventoryListener;
import eu.mhsl.craftattack.spawn.appliances.packSelect.listeners.SetPacksOnJoinListener;
import eu.mhsl.craftattack.spawn.appliances.settings.Settings;
import net.kyori.adventure.resource.ResourcePackInfo;
import net.kyori.adventure.resource.ResourcePackRequest;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.Bukkit;
import org.bukkit.NamespacedKey;
import org.bukkit.entity.Player;
import org.bukkit.event.Listener;
import org.bukkit.inventory.Inventory;
import org.bukkit.persistence.PersistentDataContainer;
import org.bukkit.persistence.PersistentDataType;
import org.jetbrains.annotations.NotNull;

import java.net.URI;
import java.util.*;

public class PackSelect extends Appliance {
    private static final NamespacedKey packKey = new NamespacedKey(Main.instance(), PackSelect.class.getName().toLowerCase(Locale.ROOT));

    public final PackConfiguration.PackList availablePacks = new PackConfiguration.PackList(new ArrayList<>());
    public final Map<Player, PackConfigurationInventory> openInventories = new WeakHashMap<>();

    public PackSelect() {
        super("packselect");
    }

    @Override
    public void onEnable() {
        Settings.instance().declareSetting(PackSelectSetting.class);

        List<Map<?, ?>> packs = localConfig().getMapList("packs");
        Bukkit.getScheduler().runTaskAsynchronously(
            Main.instance(),
            () -> packs.stream()
                .flatMap(pack -> pack.entrySet().stream())
                .forEach(pack -> {
                    @SuppressWarnings("unchecked") Map<String, String> packData = (Map<String, String>) pack.getValue();

                    try {
                        ResourcePackInfo resourcePackInfo = ResourcePackInfoFactory
                            .createResourcePackInfo(URI.create(packData.get("url")), packData.get("hash"))
                            .join();

                        PackConfiguration.Pack packToAdd = new PackConfiguration.Pack(
                            UUID.nameUUIDFromBytes(pack.getKey().toString().getBytes()),
                            packData.get("name"),
                            packData.get("description"),
                            packData.get("author"),
                            resourcePackInfo,
                            packData.get("icon")
                        );
                        availablePacks.packs().add(packToAdd);
                    } catch (Exception e) {
                        Main.logger().warning(String.format("Failed to add pack %s: %s", packData.get("name"), e.getMessage()));
                    }
                })
        );
    }

    public PackConfiguration getPackConfigurationForPlayer(Player player) {
        PersistentDataContainer persistentDataContainer = player.getPersistentDataContainer();
        try {
            String serialized = persistentDataContainer.get(packKey, PersistentDataType.STRING);
            Objects.requireNonNull(serialized);
            return PackConfiguration.deserialize(serialized);
        } catch(IllegalArgumentException | NullPointerException exception) {
            return PackConfiguration.empty();
        }
    }

    public void openPackInventory(Player player) {
        PackConfigurationInventory packInventory = new PackConfigurationInventory(getPackConfigurationForPlayer(player), player);
        player.openInventory(packInventory.getInventory());
        openInventories.put(player, packInventory);
    }

    public void setPack(Player player, PackConfiguration packConfiguration) {
        player.getPersistentDataContainer().set(packKey, PersistentDataType.STRING, packConfiguration.serialize());

        int packCount = packConfiguration.getPackList().size();
        if(packCount > 0) {
            player.sendMessage(
                Component.text(
                    String.format("%s heruntergeladen und hinzugefügt...", packCount > 1 ? "Texturenpakete werden" : "Texturenpaket wird"),
                    NamedTextColor.DARK_GREEN
                )
            );
        }

        player.sendResourcePacks(
            ResourcePackRequest.resourcePackRequest()
                .packs(
                    packConfiguration.getPackList().stream()
                        .map(PackConfiguration.Pack::info)
                        .toList()
                        .reversed()
                )
                .replace(true)
                .required(true)
                .prompt(
                    Component.text()
                        .append(Component.text("Bestätige um fortzufahren! Du kannst deine Entscheidung jederzeit mit ", NamedTextColor.GRAY))
                        .append(Component.text(String.format("/%s ", ChangePackCommand.commandName), NamedTextColor.GOLD))
                        .append(Component.text("ändern.", NamedTextColor.GRAY))
                        .build()
                )
                .build()
        );
    }

    public boolean isNotPackInventory(Player player, Inventory inventory) {
        PackConfigurationInventory packConfigurationInventory = this.openInventories.get(player);
        if(packConfigurationInventory == null) return true;
        return packConfigurationInventory.getInventory() != inventory;
    }

    @Override
    protected @NotNull List<ApplianceCommand<?>> commands() {
        return List.of(new ChangePackCommand());
    }

    @Override
    protected @NotNull List<Listener> listeners() {
        return List.of(
            new ClosePackInventoryListener(),
            new ClickPackInventoryListener(),
            new SetPacksOnJoinListener()
        );
    }
}
