package eu.mhsl.craftattack.spawn.appliances.privateMessage;

import eu.mhsl.craftattack.spawn.Main;
import eu.mhsl.craftattack.spawn.appliance.Appliance;
import eu.mhsl.craftattack.spawn.appliance.ApplianceCommand;
import eu.mhsl.craftattack.spawn.appliances.chatMessages.ChatMessages;
import eu.mhsl.craftattack.spawn.appliances.privateMessage.commands.PrivateMessageCommand;
import eu.mhsl.craftattack.spawn.appliances.privateMessage.commands.PrivateReplyCommand;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.ComponentBuilder;
import net.kyori.adventure.text.TextComponent;
import net.kyori.adventure.text.event.ClickEvent;
import net.kyori.adventure.text.event.HoverEvent;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;

import java.util.*;
import java.util.stream.Collectors;

public class PrivateMessage extends Appliance {
    public final int targetChangeTimeoutSeconds = 30;
    public final int conversationTimeoutMinutes = 30;

    private record Conversation(UUID target, Long lastSet) {}
    private final Map<Player, List<Conversation>> replyMapping = new WeakHashMap<>();

    public void reply(Player sender, String message) {
        this.replyMapping.computeIfAbsent(sender, player -> new ArrayList<>());

        List<Conversation> tooOldConversations = this.replyMapping.get(sender).stream()
            .filter(conversation -> conversation.lastSet < System.currentTimeMillis() - (conversationTimeoutMinutes*60*1000))
            .toList();
        this.replyMapping.get(sender).removeAll(tooOldConversations);

        List<Conversation> replyMap = this.replyMapping.get(sender);

        if(replyMap.isEmpty()) throw new ApplianceCommand.Error("Du führst aktuell keine Konversation.");

        Component privatePrefix = Component.text("[Privat] ", NamedTextColor.LIGHT_PURPLE);

        Conversation youngestEntry = replyMap.stream()
            .max(Comparator.comparingLong(o -> o.lastSet))
            .orElse(replyMap.getLast());

        if(message.isBlank()) {
            Player currentTargetPlayer = Bukkit.getPlayer(youngestEntry.target());

            Component currentTargetComponent = currentTargetPlayer != null
                ? Main.instance().getAppliance(ChatMessages.class).getReportablePlayerName(currentTargetPlayer)
                : Component.text("niemandem.");

            sender.sendMessage(
                privatePrefix
                    .append(Component.text("Du schreibst aktuell mit ", NamedTextColor.GRAY))
                    .append(currentTargetComponent)
            );
            return;
        }

        List<Conversation> oldConversations = replyMap.stream()
            .filter(conversation -> conversation.lastSet < System.currentTimeMillis() - (targetChangeTimeoutSeconds*1000))
            .toList();

        if(oldConversations.contains(youngestEntry) || replyMap.size() == 1) {
            Player target = Bukkit.getPlayer(youngestEntry.target());
            if(target == null) throw new ApplianceCommand.Error("Der Spieler " + Bukkit.getOfflinePlayer(youngestEntry.target()).getName() + " ist nicht mehr verfügbar.");

            this.replyMapping.get(sender).clear();
            this.sendWhisper(sender, new ResolvedPmUserArguments(target, message));
            return;
        }

        ComponentBuilder<TextComponent, TextComponent.Builder> component = Component.text();

        component.append(
            Component.newline()
                .append(privatePrefix)
                .append(Component.text("Das Ziel für /r hat sich bei dir in den letzten ", NamedTextColor.RED))
                .append(Component.text(String.valueOf(this.targetChangeTimeoutSeconds), NamedTextColor.RED))
                .append(Component.text(" Sekunden geändert. Wer soll deine Nachricht erhalten? ", NamedTextColor.RED))
                .appendNewline()
                .appendNewline()
        );

        if(!oldConversations.isEmpty()) {
            Conversation youngestOldConversation = oldConversations.stream()
                .max(Comparator.comparingLong(o -> o.lastSet))
                .orElse(oldConversations.getLast());
            this.replyMapping.get(sender).removeAll(oldConversations);
            this.replyMapping.get(sender).add(youngestOldConversation);
            replyMap = this.replyMapping.get(sender);
        }

        List<String> playerNames = replyMap.stream()
            .map(conversation -> Bukkit.getOfflinePlayer(conversation.target()).getName())
            .distinct()
            .toList();

        playerNames.forEach(playerName -> component.append(
            Component.text("[")
                .append(Component.text(playerName, NamedTextColor.GOLD))
                .append(Component.text("]"))
                .clickEvent(ClickEvent.runCommand(String.format("/msg %s %s", playerName, message)))
                .hoverEvent(HoverEvent.showText(Component.text("Klicke, um diesem Spieler zu schreiben.").color(NamedTextColor.GOLD))))
            .append(Component.text("  "))
        );
        component.appendNewline();

        sender.sendMessage(component.build());

    }

    public void sendWhisper(Player sender, ResolvedPmUserArguments userArguments) {
        Conversation newReceiverConversation = new Conversation(
            sender.getUniqueId(),
            System.currentTimeMillis()
        );

        if(this.replyMapping.get(userArguments.receiver) != null) {
            List<Conversation> oldEntries = this.replyMapping.get(userArguments.receiver).stream()
                .filter(conversation -> conversation.target() == sender.getUniqueId())
                .toList();
            this.replyMapping.get(userArguments.receiver).removeAll(oldEntries);
        } else {
            this.replyMapping.put(
                userArguments.receiver,
                new ArrayList<>()
            );
        }

        this.replyMapping.get(userArguments.receiver).add(newReceiverConversation);

        List<Conversation> senderConversationList = new ArrayList<>();
        senderConversationList.add(
            new Conversation(
                userArguments.receiver.getUniqueId(),
                System.currentTimeMillis()
            )
        );

        this.replyMapping.put(
            sender,
            senderConversationList
        );

        ChatMessages chatMessages = Main.instance().getAppliance(ChatMessages.class);
        Component privatePrefix = Component.text("[Privat] ", NamedTextColor.LIGHT_PURPLE);

        sender.sendMessage(
            Component.text()
                .append(privatePrefix.clickEvent(ClickEvent.suggestCommand(String.format("/msg %s ", userArguments.receiver.getName()))))
                .append(sender.displayName())
                .append(Component.text(" zu ", NamedTextColor.GRAY))
                .append(chatMessages.getReportablePlayerName(userArguments.receiver))
                .append(Component.text(" > ", NamedTextColor.GRAY))
                .append(Component.text(userArguments.message))
        );
        userArguments.receiver.sendMessage(
            Component.text()
                .append(privatePrefix.clickEvent(ClickEvent.suggestCommand(String.format("/msg %s ", sender.getName()))))
                .append(chatMessages.getReportablePlayerName(sender))
                .append(Component.text(" zu ", NamedTextColor.GRAY))
                .append(userArguments.receiver.displayName())
                .append(Component.text(" > ", NamedTextColor.GRAY))
                .append(Component.text(userArguments.message))
        );
    }

    public record ResolvedPmUserArguments(Player receiver, String message) {}
    public ResolvedPmUserArguments resolveImplicit(String[] args) {
        if(args.length < 2) throw new ApplianceCommand.Error("Es muss ein Spieler sowie eine Nachricht angegeben werden.");
        List<String> arguments = List.of(args);
        Player targetPlayer = Bukkit.getPlayer(arguments.getFirst());
        if(targetPlayer == null) throw new ApplianceCommand.Error(String.format("Der Spieler %s konnte nicht gefunden werden.", arguments.getFirst()));
        String message = arguments.stream().skip(1).collect(Collectors.joining(" "));
        return new ResolvedPmUserArguments(targetPlayer, message);
    }

    @Override
    protected @NotNull List<ApplianceCommand<?>> commands() {
        return List.of(
            new PrivateMessageCommand(),
            new PrivateReplyCommand()
        );
    }
}
