package eu.mhsl.craftattack.spawn.appliances.settings.datatypes;

import eu.mhsl.craftattack.spawn.appliances.settings.Settings;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.Material;
import org.bukkit.NamespacedKey;
import org.bukkit.entity.Player;
import org.bukkit.event.inventory.ClickType;
import org.bukkit.inventory.meta.ItemMeta;
import org.bukkit.persistence.PersistentDataContainer;
import org.bukkit.persistence.PersistentDataType;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.IntStream;

public abstract class SelectSetting extends Setting<SelectSetting.Options.Option> {
    private final Options options;

    public SelectSetting(Settings.Key key, Options options) {
        super(key);
        this.options = options;
    }

    public record Options(List<Option> options) {
        public record Option(String name, NamespacedKey key) {
            public boolean is(Option other) {
                return this.equals(other);
            }
        }
    }

    protected abstract Material icon();
    protected abstract Options.Option defaultValue();

    @Override
    public ItemMeta buildMeta(ItemMeta meta) {
        meta.displayName(Component.text(title(), NamedTextColor.WHITE));
        List<Component> lore = new ArrayList<>();
        lore.add(Component.text("Status: ", NamedTextColor.DARK_GRAY));
        lore.addAll(
            this.options.options.stream()
                .map(option -> {
                    boolean isSelected = option.equals(this.state);
                    return Component.text()
                        .append(Component.text(isSelected ? "> " : "  ", isSelected ? NamedTextColor.GREEN : NamedTextColor.GRAY))
                        .append(Component.text(option.name, isSelected ? NamedTextColor.DARK_GREEN : NamedTextColor.DARK_GRAY))
                        .build();
                })
                .toList()
        );
        lore.add(Component.empty());
        lore.addAll(buildDescription(description()));
        meta.lore(lore);
        return meta;
    }

    @Override
    protected void change(Player player, ClickType clickType) {
        int optionModifier = clickType.equals(ClickType.LEFT) ? 1 : -1;
        List<Options.Option> options = this.options.options;
        this.state = IntStream.range(0, options.size())
            .filter(i -> options.get(i).equals(this.state))
            .mapToObj(i -> {
                int nextIndex = i + optionModifier;
                if (nextIndex >= options.size()) {
                    return options.getFirst();
                } else if (nextIndex < 0) {
                    return options.getLast();
                } else {
                    return options.get(nextIndex);
                }
            })
            .findFirst()
            .orElseThrow();
    }

    @Override
    protected void fromStorage(PersistentDataContainer container) {
        String data = container.has(getNamespacedKey())
            ? Objects.requireNonNull(container.get(getNamespacedKey(), PersistentDataType.STRING))
            : defaultValue().key.asString();

        this.state = this.options.options.stream()
            .filter(option -> option.key.asString().equals(data))
            .findFirst()
            .orElse(defaultValue());
    }

    @Override
    protected void toStorage(PersistentDataContainer container, Options.Option value) {
        container.set(getNamespacedKey(), PersistentDataType.STRING, value.key.asString());
    }

    @Override
    public Class<?> dataType() {
        return Options.Option.class;
    }

    @Override
    public Options.Option state() {
        return this.state;
    }
}
