package eu.mhsl.craftattack.spawn.appliances.tablist;

import eu.mhsl.craftattack.spawn.Main;
import eu.mhsl.craftattack.spawn.appliance.Appliance;
import eu.mhsl.craftattack.spawn.appliances.report.Report;
import eu.mhsl.craftattack.spawn.appliances.settings.Settings;
import eu.mhsl.craftattack.spawn.util.IteratorUtil;
import eu.mhsl.craftattack.spawn.util.statistics.NetworkMonitor;
import eu.mhsl.craftattack.spawn.util.text.ComponentUtil;
import eu.mhsl.craftattack.spawn.util.text.RainbowComponent;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import net.kyori.adventure.util.Ticks;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.bukkit.event.Listener;
import org.jetbrains.annotations.NotNull;

import java.lang.management.ManagementFactory;
import java.lang.management.OperatingSystemMXBean;
import java.time.Duration;
import java.util.List;


public class Tablist extends Appliance {
    private final RainbowComponent serverName = new RainbowComponent(" CraftAttack 7 ", 7, 3);
    private NetworkMonitor networkMonitor;
    private OperatingSystemMXBean systemMonitor;

    public Tablist() {
        super("tablist");
    }

    @Override
    public void onEnable() {
        Settings.instance().declareSetting(TechnicalTablistSetting.class);

        int tabRefreshRate = 3;
        this.networkMonitor = new NetworkMonitor(localConfig().getString("interface"), Duration.ofSeconds(1));
        this.systemMonitor = ManagementFactory.getOperatingSystemMXBean();

        Bukkit.getScheduler().runTaskTimerAsynchronously(
            Main.instance(),
            () -> IteratorUtil.onlinePlayers(this::updateHeader),
            tabRefreshRate * Ticks.TICKS_PER_SECOND,
            tabRefreshRate * Ticks.TICKS_PER_SECOND
        );
    }

    @Override
    public void onDisable() {
        this.networkMonitor.stop();
    }

    public void fullUpdate(Player player) {
        updateHeader(player);
        updateFooter(player);
    }

    private void updateHeader(Player player) {
        boolean detailedInfo = queryAppliance(Settings.class).getSetting(player, Settings.Key.TechnicalTab, Boolean.class);
        Component header = Component.newline()
            .append(serverName.getRainbowState()).appendNewline()
            .append(Component.text("mhsl.eu", NamedTextColor.GOLD)).appendNewline().appendNewline()
            .append(ComponentUtil.getFormattedTickTimes(detailedInfo)).appendNewline();

        if(detailedInfo) {
            header = header
                .appendNewline()
                .append(ComponentUtil.getFormattedPing(player)).appendNewline()
                .append(ComponentUtil.getFormattedNetworkStats(
                    this.networkMonitor.getTraffic(),
                    this.networkMonitor.getPackets()
                )).appendNewline()
                .append(ComponentUtil.getFormattedSystemStats(this.systemMonitor)).appendNewline();
        }

        player.sendPlayerListHeader(header);
    }

    private void updateFooter(Player player) {
        player.sendPlayerListFooter(Report.helpText());
    }

    @Override
    @NotNull
    protected List<Listener> listeners() {
        return List.of(new TablistListener());
    }
}
