package eu.mhsl.craftattack.spawn.appliances.whitelist;

import com.google.gson.Gson;
import eu.mhsl.craftattack.spawn.Main;
import eu.mhsl.craftattack.spawn.api.HttpServer;
import eu.mhsl.craftattack.spawn.appliance.Appliance;
import eu.mhsl.craftattack.spawn.appliances.outlawed.Outlawed;
import eu.mhsl.craftattack.spawn.util.server.Floodgate;
import eu.mhsl.craftattack.spawn.util.text.DisconnectInfo;
import org.apache.http.client.utils.URIBuilder;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.bukkit.event.Listener;
import org.jetbrains.annotations.NotNull;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.time.Instant;
import java.time.ZoneOffset;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.UUID;
import java.util.logging.Level;

public class Whitelist extends Appliance {
    private record UserData(UUID uuid, String username, String firstname, String lastname, Long banned_until,
                            Long outlawed_until) {
    }

    private final URI apiEndpoint = URI.create(Objects.requireNonNull(localConfig().getString("api")));

    public Whitelist() {
        super("whitelist");
    }

    public void integrityCheck(Player player) throws DisconnectInfo.Throwable {
        try {
            Main.instance().getLogger().info(String.format("Running integrityCheck for %s", player.getName()));
            boolean overrideCheck = localConfig().getBoolean("overrideIntegrityCheck", false);
            UserData user = overrideCheck
                ? new UserData(player.getUniqueId(), player.getName(), "", "", 0L, 0L)
                : this.fetchUserData(player.getUniqueId());

            if(timestampRelevant(user.banned_until)) {
                Instant bannedDate = new Date(user.banned_until * 1000L)
                    .toInstant()
                    .plus(1, ChronoUnit.HOURS);

                DateTimeFormatter dateFormat = DateTimeFormatter.ofPattern("dd.MM.yyyy").withZone(ZoneOffset.UTC);
                DateTimeFormatter timeFormat = DateTimeFormatter.ofPattern("HH:mm").withZone(ZoneOffset.UTC);

                throw new DisconnectInfo.Throwable(
                    "Du wurdest vom Server gebannt.",
                    String.format("Dein Bann läuft am %s um %s ab!", dateFormat.format(bannedDate), timeFormat.format(bannedDate)),
                    "Wende dich an einen Admin für weitere Informationen.",
                    player.getUniqueId()
                );
            }

            queryAppliance(Outlawed.class).updateForcedStatus(player, timestampRelevant(user.outlawed_until));

            String purePlayerName = Floodgate.isBedrock(player)
                ? Floodgate.getBedrockPlayer(player).getUsername()
                : player.getName();

            if(!user.username.trim().equalsIgnoreCase(purePlayerName))
                throw new DisconnectInfo.Throwable(
                    "Nutzername geändert",
                    String.format("Der Name '%s' stimmt nicht mit '%s' überein.", user.username, player.getName()),
                    "Bitte kontaktiere einen Admin, um Deine Anmeldedaten zu aktualisieren!",
                    player.getUniqueId()
                );

        } catch(DisconnectInfo.Throwable e) {
            throw e;
        } catch(Exception e) {
            Main.instance().getLogger().log(Level.SEVERE, e, e::getMessage);
            throw new DisconnectInfo.Throwable(
                "Interner Serverfehler",
                "Deine Anmeldedaten konnten nicht abgerufen/ überprüft werden.",
                "Versuche es später erneut oder kontaktiere einen Admin!",
                player.getUniqueId()
            );
        }
    }

    private boolean timestampRelevant(Long timestamp) {
        if(timestamp == null) return false;
        return timestamp > System.currentTimeMillis() / 1000L;
    }

    private UserData fetchUserData(UUID uuid) throws DisconnectInfo.Throwable {
        URIBuilder uriBuilder = new URIBuilder(apiEndpoint);
        uriBuilder.addParameter("uuid", uuid.toString());

        try(HttpClient client = HttpClient.newHttpClient()) {
            HttpRequest httpRequest = HttpRequest.newBuilder()
                .uri(uriBuilder.build())
                .header("Content-Type", "application/json")
                .GET()
                .build();

            HttpResponse<String> httpResponse = client.send(httpRequest, HttpResponse.BodyHandlers.ofString());

            if(httpResponse.statusCode() == 404)
                throw new DisconnectInfo.Throwable(
                    "Nicht angemeldet",
                    "Du bist derzeit nicht als Teilnehmer des CraftAttack-Projektes registriert!",
                    "Melde Dich bei einem Admin für eine nachträgliche Anmeldung.",
                    uuid
                );

            return new Gson().fromJson(httpResponse.body(), UserData.class);

        } catch(IOException | InterruptedException | URISyntaxException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public void httpApi(HttpServer.ApiBuilder apiBuilder) {
        record User(UUID user) {
        }
        apiBuilder.post("update", User.class, (user, request) -> {
            Main.instance().getLogger().info(String.format("API Triggered Profile update for %s", user.user));
            Player player = Bukkit.getPlayer(user.user);
            if(player != null) {
                try {
                    this.integrityCheck(player);
                } catch(DisconnectInfo.Throwable e) {
                    e.getDisconnectScreen().applyKick(player);
                }
            }
            return HttpServer.nothing;
        });
    }

    @Override
    @NotNull
    protected List<Listener> listeners() {
        return List.of(
            new PlayerJoinListener()
        );
    }
}
