package eu.mhsl.craftattack.spawn.util.inventory;

import eu.mhsl.craftattack.spawn.util.text.ComponentUtil;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.Material;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;
import org.jetbrains.annotations.Contract;
import org.jetbrains.annotations.NotNull;

import java.util.List;
import java.util.Objects;
import java.util.function.Consumer;
import java.util.function.Function;

public class ItemBuilder {
    private final ItemStack itemStack;

    @Contract(value = "_ -> new", pure = true)
    public static @NotNull ItemBuilder of(Material material) {
        return new ItemBuilder(material);
    }

    @Contract(value = "_ -> new", pure = true)
    public static @NotNull ItemBuilder of(ItemStack itemStack) {
        return new ItemBuilder(itemStack);
    }

    private ItemBuilder(Material material) {
        this.itemStack = ItemStack.of(material);
    }

    private ItemBuilder(ItemStack itemStack) {
        this.itemStack = itemStack;
    }

    public ItemBuilder displayName(Component displayName) {
        return this.withMeta(itemMeta -> itemMeta.displayName(displayName));
    }

    public ItemBuilder displayName(Function<Component, Component> process) {
        return this.displayName(process.apply(itemStack.displayName()));
    }

    public ItemBuilder lore(String text) {
        return this.lore(text, 50, NamedTextColor.GRAY);
    }

    public ItemBuilder lore(String text, NamedTextColor color) {
        return this.lore(text, 50, color);
    }

    public ItemBuilder lore(String text, int linebreak, NamedTextColor color) {
        return this.withMeta(itemMeta -> itemMeta.lore(
            ComponentUtil.lineBreak(text, linebreak)
                .map(s -> Component.text(s, color))
                .toList()
        ));
    }

    public ItemBuilder appendLore(Component text) {
        List<Component> lore = itemStack.lore();
        Objects.requireNonNull(lore, "Cannot append lore to Item without lore");
        lore.add(text);
        return this.withMeta(itemMeta -> itemMeta.lore(lore));
    }

    public ItemBuilder noStacking() {
        return this.withMeta(itemMeta -> itemMeta.setMaxStackSize(1));
    }

    public ItemBuilder glint() {
        return this.withMeta(itemMeta -> itemMeta.setEnchantmentGlintOverride(true));
    }

    public ItemBuilder amount(int amount) {
        this.itemStack.setAmount(amount);
        return this;
    }

    public ItemBuilder withMeta(Consumer<ItemMeta> callback) {
        ItemMeta meta = this.itemStack.getItemMeta();
        callback.accept(meta);
        this.itemStack.setItemMeta(meta);
        return this;
    }

    public ItemStack build() {
        return this.itemStack;
    }
}
